#include "StdAfx.h"
#include "ImageWindow.h"

extern HINSTANCE g_hInstance;
static const wchar_t * g_pstrClassName = L"ImageResizerImageWindow";
static const wchar_t * g_pstrWindowObject = L"ImageWindowObject";

//////////////////////////////////////
//// Construction/Destruction
//////////////////////////////////////

ImageWindow::ImageWindow() throw() : Handle(NULL),m_hBitmap(NULL),Width(0),Height(0)
{
}

ImageWindow::~ImageWindow() throw()
{
    if (m_hBitmap != NULL)
    {
        ::DeleteObject(m_hBitmap);
    }
}

//////////////////////////////////////
//// Member Functions
//////////////////////////////////////

void ImageWindow::LoadBitmap(const wchar_t * pstrFileName) throw()
{
    if (m_hBitmap != NULL)
    {
        ::DeleteObject(m_hBitmap);
    }
    m_hBitmap = (HBITMAP)::LoadImageW(NULL,pstrFileName,IMAGE_BITMAP,0,0,LR_DEFAULTSIZE | LR_LOADFROMFILE);
    Invalidate(true);
}

void ImageWindow::SaveBitmap(const wchar_t * pstrFileName) throw()
{
    if (m_hBitmap != NULL)
    {
        BITMAP bmpData;
        
        ::GetObject(m_hBitmap,sizeof(BITMAP),&bmpData);
        
        BITMAPFILEHEADER theFileHeader;
        theFileHeader.bfType = 'MB';
        theFileHeader.bfSize = sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER) + (Width * Height * 4);
        theFileHeader.bfReserved1 = 0;
        theFileHeader.bfReserved2 = 0;
        theFileHeader.bfOffBits = sizeof(BITMAPINFO);
        
        BITMAPINFO theInfo;
        ::memset(&theInfo,0,sizeof(BITMAPINFO));
        
        theInfo.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
        theInfo.bmiHeader.biSizeImage = Width * Height * 4;
        theInfo.bmiHeader.biBitCount = bmpData.bmBitsPixel;
        theInfo.bmiHeader.biPlanes = 1;
        theInfo.bmiHeader.biWidth = Width;
        theInfo.bmiHeader.biHeight = Height;
        theInfo.bmiHeader.biXPelsPerMeter = 10000;
        theInfo.bmiHeader.biYPelsPerMeter = 10000;
        
        HANDLE hFile = ::CreateFileW(pstrFileName,GENERIC_WRITE,FILE_SHARE_READ | FILE_SHARE_WRITE,
            NULL,CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL,NULL);
            
        if (hFile != INVALID_HANDLE_VALUE)
        {
            if (::SetFilePointer(hFile,theInfo.bmiHeader.biSizeImage,0,FILE_BEGIN) != 0xFFFFFFFF)
            {
                ::SetEndOfFile(hFile);
                ::SetFilePointer(hFile,0,0,FILE_BEGIN);
                
                unsigned long dwBytesWritten(0);
                ::WriteFile(hFile,&theFileHeader,sizeof(BITMAPFILEHEADER),&dwBytesWritten,NULL);
                ::WriteFile(hFile,&theInfo,sizeof(BITMAPINFO),&dwBytesWritten,NULL);
                
                HDC hWindowDC = ::GetDC(Handle);
                HBITMAP hDIBSection = ::CreateCompatibleBitmap(hWindowDC,Width,Height);
                
                HDC hOutputDC = ::CreateCompatibleDC(hWindowDC);
                HDC hMemDC = ::CreateCompatibleDC(hWindowDC);
                ::ReleaseDC(Handle,hWindowDC);
                
                HBITMAP hOldBitmap = (HBITMAP)::SelectObject(hMemDC,m_hBitmap);
                HBITMAP hOldBitmap2 = (HBITMAP)::SelectObject(hOutputDC,hDIBSection);
                
                ::StretchBlt(hOutputDC,0,0,Width,Height,hMemDC,0,0,bmpData.bmWidth,bmpData.bmHeight,SRCCOPY);
                
                void * lpBits = ::HeapAlloc(::GetProcessHeap(),HEAP_ZERO_MEMORY,theInfo.bmiHeader.biSizeImage);
                
                ::GetDIBits(hOutputDC,hDIBSection,0,Height,lpBits,&theInfo,DIB_RGB_COLORS);
                
                if (bmpData.bmBitsPixel == 32)
                {
                    unsigned long * pColors = reinterpret_cast<unsigned long *>(lpBits);
                    unsigned long dwTempValue(0),dwTempValue2(0);
                    const unsigned long dwSize = Width * Height;
                    for (unsigned long x = 0; x < dwSize; ++x)
                    {
                        dwTempValue = pColors[x] & 0x00FF0000;
                        dwTempValue2 = pColors[x] & 0x000000FF;
                        pColors[x] ^= (dwTempValue | dwTempValue2);
                        pColors[x] |= (dwTempValue >> 16);
                        pColors[x] |= (dwTempValue2 << 16);
                    }
                }
                
                ::WriteFile(hFile,lpBits,theInfo.bmiHeader.biSizeImage,&dwBytesWritten,NULL);
                
                ::HeapFree(::GetProcessHeap(),0,lpBits);
                
                ::CloseHandle(hFile);
                
                ::SelectObject(hOutputDC,hOldBitmap2);
                ::SelectObject(hMemDC,hOldBitmap);
                
                ::DeleteObject(hDIBSection);
                ::DeleteDC(hMemDC);
                ::DeleteDC(hOutputDC);
            }
            else
            {
                ::CloseHandle(hFile);
                ::MessageBoxW(Handle,L"Unable to save bitmap to file.",L"Image Resizer",MB_ICONERROR | MB_OK);
            }
        }
    }
}

bool ImageWindow::Create(const wchar_t * pstrTitle,unsigned long dwStyle,
    unsigned long dwExtendedStyle,long x,long y,long nWidth,long nHeight,HWND hParentWnd) throw()
{
    Handle = ::CreateWindowExW(dwExtendedStyle,::g_pstrClassName,pstrTitle,
        dwStyle,x,y,nWidth,nHeight,hParentWnd,NULL,NULL,this);
        
    return Handle != NULL;
}

bool ImageWindow::Show(int nShow) throw()
{
    return ::ShowWindow(Handle,nShow) != 0;
}

bool ImageWindow::Invalidate(bool bErase) throw()
{
    const BOOL bEraseBk = bErase == true ? TRUE : FALSE;
    return ::InvalidateRect(Handle,NULL,bEraseBk) != 0;
}

void ImageWindow::Close() throw()
{
    ::SendMessageW(Handle,WM_CLOSE,0,0);
}

//////////////////////////////////////
//// Window Registration
//////////////////////////////////////

bool ImageWindow::RegisterWindow() throw()
{
    WNDCLASSEXW wndClass;
    
    wndClass.cbSize = sizeof(WNDCLASSEXW);
    wndClass.style = CS_DBLCLKS;
    wndClass.hInstance = ::g_hInstance;
    wndClass.lpfnWndProc = reinterpret_cast<WNDPROC>(ImageWindow::HandleWindowProcess);
    wndClass.hbrBackground = NULL;
    wndClass.hCursor = ::LoadCursor(NULL,IDC_ARROW);
    wndClass.hIcon = ::LoadIcon(NULL,IDI_APPLICATION);
    wndClass.hIconSm = wndClass.hIcon;
    wndClass.cbClsExtra = 0;
    wndClass.cbWndExtra = 0;
    wndClass.lpszMenuName = NULL;
    wndClass.lpszClassName = ::g_pstrClassName;
    
    return ::RegisterClassExW(&wndClass) != 0;
}

bool ImageWindow::UnregisterWindow() throw()
{
    return ::UnregisterClassW(::g_pstrClassName,::g_hInstance) != 0;
}

//////////////////////////////////////
//// Window Process
//////////////////////////////////////

INT_PTR ImageWindow::WindowProcess(UINT message,WPARAM wParam,LPARAM lParam) throw()
{
    if (message == WM_NCDESTROY)
    {
        delete this;
        return 0;
    }
    else if (message == WM_PAINT)
    {
        PAINTSTRUCT ps;
        HDC hPaintDC = ::BeginPaint(Handle,&ps);
        OnPaint(hPaintDC);
        ::EndPaint(Handle,&ps);
        return 0;
    }
    else if (message == WM_SIZE)
    {
        OnSize(wParam,lParam);
        return 0;
    }
    return ::DefWindowProcW(Handle,message,wParam,lParam);
}

INT_PTR __stdcall ImageWindow::HandleWindowProcess(HWND hWnd,UINT message,WPARAM wParam,LPARAM lParam) throw()
{
    if (message == WM_NCCREATE)
    {
        LPCREATESTRUCTW lpCreate = reinterpret_cast<LPCREATESTRUCTW>(lParam);
        ImageWindow * pWindow = reinterpret_cast<ImageWindow *>(lpCreate->lpCreateParams);
        pWindow->Handle = hWnd;
        pWindow->Width = lpCreate->cx;
        pWindow->Height = lpCreate->cy;
        ::SetPropW(hWnd,::g_pstrWindowObject,reinterpret_cast<HANDLE>(pWindow));
        return pWindow->WindowProcess(message,wParam,lParam);
    }
    else
    {
        ImageWindow * pWindow = reinterpret_cast<ImageWindow *>(::GetPropW(hWnd,::g_pstrWindowObject));
        if (pWindow != NULL)
        {
            return pWindow->WindowProcess(message,wParam,lParam);
        }
        else
        {
            return ::DefWindowProcW(hWnd,message,wParam,lParam);
        }
    }
}

//////////////////////////////////////
//// Message Handlers
//////////////////////////////////////

void ImageWindow::OnPaint(HDC hClientDC) throw()
{
    
    if (m_hBitmap != NULL)
    {
        BITMAP theData;
        
        ::GetObject(m_hBitmap,sizeof(BITMAP),&theData);
        
        const long nBitmapWidth = theData.bmWidth;
        const long nBitmapHeight = theData.bmHeight;
        
        HDC hMemDC = ::CreateCompatibleDC(hClientDC);
        HBITMAP hOldBitmap = reinterpret_cast<HBITMAP>(::SelectObject(hMemDC,m_hBitmap));
        
        ::StretchBlt(hClientDC,0,0,Width,Height,hMemDC,0,0,nBitmapWidth,nBitmapHeight,SRCCOPY);
        
        ::SelectObject(hMemDC,hOldBitmap);
        ::DeleteDC(hMemDC);
    }
}

void ImageWindow::OnSize(WPARAM wParam,LPARAM lParam) throw()
{
    if (wParam == SIZE_RESTORED || wParam == SIZE_MAXIMIZED)
    {
        Width = LOWORD(lParam);
        Height = HIWORD(lParam);
        wchar_t szText[128];
        ::wsprintfW(szText,L"Width: %lu Height: %lu",Width,Height);
        ::SetWindowTextW(StatusWindow,szText);
        Invalidate(true);
    }
}
