#include "StdAfx.h"
#include "MainWindow.h"
#include "HelpWindow.h"
#include "resource.h"

extern HINSTANCE g_hInstance;
static const wchar_t * g_pstrClassName = L"ImageResizerMainWindow";
static const wchar_t * g_pstrWindowObject = L"WindowObject";

//////////////////////////////////////
//// Construction/Destruction
//////////////////////////////////////

MainWindow::MainWindow() throw() : Handle(NULL),MenuHandle(NULL),ImageObject(0),StatusWindow(NULL)
{
    MenuHandle = ::LoadMenuW(::g_hInstance,MAKEINTRESOURCEW(IDR_MAINFRAME));
    ImageObject = new ImageWindow();
}

MainWindow::~MainWindow() throw()
{
    ::DestroyMenu(MenuHandle);
}

//////////////////////////////////////
//// Member Functions
//////////////////////////////////////

bool MainWindow::Create(const wchar_t * pstrTitle,unsigned long dwStyle,
    unsigned long dwExtendedStyle,long x,long y,long nWidth,long nHeight) throw()
{
    Handle = ::CreateWindowExW(dwExtendedStyle,::g_pstrClassName,pstrTitle,
        dwStyle,x,y,nWidth,nHeight,NULL,MenuHandle,NULL,this);
        
    return Handle != NULL;
}

bool MainWindow::Show(int nShow) throw()
{
    return ::ShowWindow(Handle,nShow) != 0;
}

bool MainWindow::Invalidate(bool bErase) throw()
{
    const BOOL bEraseBk = bErase == true ? TRUE : FALSE;
    return ::InvalidateRect(Handle,NULL,bEraseBk) != 0;
}

//////////////////////////////////////
//// Window Registration
//////////////////////////////////////

bool MainWindow::RegisterWindow() throw()
{
    WNDCLASSEXW wndClass;
    
    wndClass.cbSize = sizeof(WNDCLASSEXW);
    wndClass.style = CS_DBLCLKS;
    wndClass.hInstance = ::g_hInstance;
    wndClass.lpfnWndProc = reinterpret_cast<WNDPROC>(MainWindow::HandleWindowProcess);
    wndClass.hbrBackground = ::CreateSolidBrush(RGB(195,195,209));
    wndClass.hCursor = ::LoadCursor(NULL,IDC_ARROW);
    wndClass.hIcon = ::LoadIcon(NULL,IDI_APPLICATION);
    wndClass.hIconSm = wndClass.hIcon;
    wndClass.cbClsExtra = 0;
    wndClass.cbWndExtra = 0;
    wndClass.lpszMenuName = NULL;
    wndClass.lpszClassName = ::g_pstrClassName;
    
    return ::RegisterClassExW(&wndClass) != 0;
}

bool MainWindow::UnregisterWindow() throw()
{
    return ::UnregisterClassW(::g_pstrClassName,::g_hInstance) != 0;
}

//////////////////////////////////////
//// Window Process
//////////////////////////////////////

INT_PTR MainWindow::WindowProcess(UINT message,WPARAM wParam,LPARAM lParam) throw()
{
    if (message == WM_CREATE)
    {
        return OnCreate(reinterpret_cast<LPCREATESTRUCTW>(lParam));
    }
    else if (message == WM_COMMAND)
    {
        return OnCommand(wParam,lParam);
    }
    else if (message == WM_SIZE)
    {
        OnSize(wParam,lParam);
        return 0;
    }
    else if (message == WM_CLOSE)
    {
        OnClose();
        return 0;
    }
    else if (message == WM_NCDESTROY)
    {
        delete this;
        ::PostQuitMessage(0);
        return 0;
    }
    return ::DefWindowProcW(Handle,message,wParam,lParam);
}

INT_PTR __stdcall MainWindow::HandleWindowProcess(HWND hWnd,UINT message,WPARAM wParam,LPARAM lParam) throw()
{
    if (message == WM_NCCREATE)
    {
        LPCREATESTRUCTW lpCreate = reinterpret_cast<LPCREATESTRUCTW>(lParam);
        MainWindow * pWindow = reinterpret_cast<MainWindow *>(lpCreate->lpCreateParams);
        pWindow->Handle = hWnd;
        pWindow->Width = lpCreate->cx;
        pWindow->Height = lpCreate->cy;
        ::SetPropW(hWnd,::g_pstrWindowObject,reinterpret_cast<HANDLE>(pWindow));
        return pWindow->WindowProcess(message,wParam,lParam);
    }
    else
    {
        MainWindow * pWindow = reinterpret_cast<MainWindow *>(::GetPropW(hWnd,::g_pstrWindowObject));
        if (pWindow != NULL)
        {
            return pWindow->WindowProcess(message,wParam,lParam);
        }
        else
        {
            return ::DefWindowProcW(hWnd,message,wParam,lParam);
        }
    }
}

//////////////////////////////////////
//// Message Handlers
//////////////////////////////////////

INT_PTR MainWindow::OnCreate(LPCREATESTRUCTW lpCreate) throw()
{
    ImageObject->Create(L"",WS_CHILD | WS_SIZEBOX,WS_EX_STATICEDGE,0,0,640,480,Handle);
    ImageObject->Show(SW_SHOW);
    StatusWindow = ::CreateStatusWindowW(WS_CHILD | WS_VISIBLE,L"Ready...",Handle,1000);
    ImageObject->StatusWindow = StatusWindow;
    return 0;
}

INT_PTR MainWindow::OnCommand(WPARAM wParam,LPARAM lParam) throw()
{
    const unsigned short wNotification = HIWORD(wParam);
    const unsigned short wID = LOWORD(wParam);
    if (wNotification == BN_CLICKED)
    {
        if (wID == IDM_FILE_OPEN)
        {
            OnFileOpen();
            return 0;
        }
        else if (wID == IDM_FILE_SAVE)
        {
            OnFileSave();
            return 0;
        }
        else if (wID == IDM_FILE_EXIT)
        {
            OnFileExit();
            return 0;
        }
        else if (wID == IDM_HELP_ABOUT)
        {
            HelpWindow::Show();
            return 0;
        }
    }
    return ::DefWindowProcW(Handle,WM_COMMAND,wParam,lParam);
}

void MainWindow::OnSize(WPARAM wParam,LPARAM lParam) throw()
{
    if (wParam == SIZE_RESTORED || wParam == SIZE_MAXIMIZED)
    {
        Width = LOWORD(lParam);
        Height = HIWORD(lParam);
        ::MoveWindow(StatusWindow,0,Height - 25,Width,25,TRUE);
    }
}

void MainWindow::OnClose() throw()
{
    ImageObject->Close();
    
    ::DestroyWindow(Handle);
}

//////////////////////////////////////
//// Command Handlers
//////////////////////////////////////

void MainWindow::OnFileOpen() throw()
{
    wchar_t * pstrPathName = new wchar_t[4096];
    wchar_t * pstrFileTitle = new wchar_t[128];
    
    OPENFILENAMEW ofn;
    ofn.lStructSize = sizeof(OPENFILENAMEW);
    ofn.hwndOwner = Handle;
    ofn.hInstance = NULL;
    ofn.lpstrFilter = L"Bitmaps (*.bmp)\0*.bmp\0";
    ofn.lpstrCustomFilter = NULL;
    ofn.nMaxCustFilter = 0;
    ofn.nFilterIndex = 0;
    ofn.lpstrFile = pstrPathName;
    ofn.nMaxFile = 4096;
    ofn.lpstrFileTitle = pstrFileTitle;
    ofn.nMaxFileTitle = 128;
    ofn.lpstrInitialDir = NULL;
    ofn.lpstrTitle = NULL;
    ofn.Flags = OFN_HIDEREADONLY | OFN_FILEMUSTEXIST;
    ofn.nFileOffset = 0;
    ofn.nFileExtension = 0;
    ofn.lpstrDefExt = L"bmp";
    ofn.lCustData = 0;
    ofn.lpfnHook = NULL;
    ofn.lpTemplateName = NULL;
    
    if (::GetOpenFileNameW(&ofn) != 0)
    {
        ImageObject->LoadBitmap(pstrPathName);
        
        MENUITEMINFO info;
        info.cbSize = sizeof(MENUITEMINFO);
        info.fMask = MIIM_STATE;
        info.fState = MFS_UNHILITE;
        
        ::SetMenuItemInfo(MenuHandle,IDM_FILE_SAVE,0,&info);
    }
    
    delete [] pstrPathName;
    delete [] pstrFileTitle;
}

void MainWindow::OnFileSave() throw()
{
    wchar_t * pstrPathName = new wchar_t[4096];
    wchar_t * pstrFileTitle = new wchar_t[128];
    
    OPENFILENAMEW ofn;
    ofn.lStructSize = sizeof(OPENFILENAMEW);
    ofn.hwndOwner = Handle;
    ofn.hInstance = NULL;
    ofn.lpstrFilter = L"Bitmaps (*.bmp)\0*.bmp\0";
    ofn.lpstrCustomFilter = NULL;
    ofn.nMaxCustFilter = 0;
    ofn.nFilterIndex = 0;
    ofn.lpstrFile = pstrPathName;
    ofn.nMaxFile = 4096;
    ofn.lpstrFileTitle = pstrFileTitle;
    ofn.nMaxFileTitle = 128;
    ofn.lpstrInitialDir = NULL;
    ofn.lpstrTitle = NULL;
    ofn.Flags = OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT;
    ofn.nFileOffset = 0;
    ofn.nFileExtension = 0;
    ofn.lpstrDefExt = L"bmp";
    ofn.lCustData = 0;
    ofn.lpfnHook = NULL;
    ofn.lpTemplateName = NULL;
    
    if (::GetSaveFileNameW(&ofn) != 0)
    {
        ImageObject->SaveBitmap(pstrPathName);
    }
    
    delete [] pstrPathName;
    delete [] pstrFileTitle;
}

void MainWindow::OnFileExit() throw()
{
    ::SendMessageW(Handle,WM_CLOSE,0,0);
}
